@@ -18,6 +18,9 @@ our (%gui);
         invalidipv4address => ['Invalid IP Address',
                                'An invalid IP address has been entered.',
                                'error'],
+        invalidipv6address => ['Invalid IPv6 Address',
+                               'An invalid IPv6 address has been entered.',
+                               'error'],
         invalidipv4netmask => ['Invalid IP Netmask',
                                'An invalid IP netmask has been entered.',
                                'error'],
@@ -28,8 +31,8 @@ our (%gui);
                           'Credentials are incorrect or the VirtualBox Web Service is not running at the specified URL.',
                           'error'],
         applimport    => ['Failed Appliance Import',
-                          'The appliance failed to import correctly. This may be due to a corrupt or incompatible' .
-                          " OVF/OVA file or a permission error\n",
+                          'The appliance failed to import correctly. The file may be missing, corrupt, have incorrect' .
+                          " permissions or not in OVF/OVA format\n",
                           'error'],
         createguest   => ['Failed to Create Guest',
                           'Verify your guest settings. Cause may be a duplicate guest name.',
