@@ -180,6 +180,7 @@ sub storage_ctr_add() {
     elsif ($widget eq $gui{menuitemCtrAddSCSI}) { $bus = 'SCSI'; }
     elsif ($widget eq $gui{menuitemCtrAddSATA}) { $bus = 'SATA'; }
     elsif ($widget eq $gui{menuitemCtrAddSAS}) { $bus = 'SAS'; }
+    elsif ($widget eq $gui{menuitemCtrAddUSB}) { $bus = 'USB'; }
 
     my @Controllers = IMachine_getStorageControllers($vmc{IMachine});
     my $exists = 0;
@@ -188,6 +189,9 @@ sub storage_ctr_add() {
 
     if (!$exists) {
         my $IStorageController = IMachine_addStorageController($vmc{IMachine}, "$bus Controller", $bus);
+        # This works around the controller type defaults defaulting to USB. VB itself doesn't seem to use it.
+        if ($bus eq 'SAS') { IStorageController_setControllerType($IStorageController, 'LsiLogicSas', 0); }
+
         IStorageController_setPortCount($IStorageController, IStorageController_getMinPortCount($IStorageController)); # Controllers have all ports on by default. Set to the minimum
         &fill_list_editstorage($vmc{IMachine});
     }
