@@ -5,9 +5,9 @@ use File::Spec;
 use File::Spec::Functions;
 use File::Spec::Win32;
 use File::Spec::Unix;
-use Gtk2 -init;
+use Getopt::Std;
+use Gtk2 '-init';
 our $sharedir;
-our $spazz;
 
 my $builder = Gtk2::Builder->new;
 $builder->add_from_file("$sharedir/remotebox.xml");
@@ -22,8 +22,8 @@ our %gui = (textbufferEditGenDescription         => $builder->get_object('textbu
             adjustmentEditSysProcessor           => $builder->get_object('adjustmentEditSysProcessor'),
             adjustmentEditSysProcessorCap        => $builder->get_object('adjustmentEditSysProcessorCap'),
             adjustmentEditStorPortCount          => $builder->get_object('adjustmentEditStorPortCount'),
-            adjustmentPFIPv4HostPort             => $builder->get_object('adjustmentPFIPv4HostPort'),
-            adjustmentPFIPv4GuestPort            => $builder->get_object('adjustmentPFIPv4GuestPort'),
+            adjustmentPFIPHostPort               => $builder->get_object('adjustmentPFIPHostPort'),
+            adjustmentPFIPGuestPort              => $builder->get_object('adjustmentPFIPGuestPort'),
             appname                              => $builder->get_object('aboutdialog')->get_program_name(),
             appver                               => $builder->get_object('aboutdialog')->get_version(),
             vboxEditIOPorts                      => $builder->get_object('vboxEditIOPorts'),
@@ -31,30 +31,34 @@ our %gui = (textbufferEditGenDescription         => $builder->get_object('textbu
 
 # Guest Status
 $gui{img}{Aborted} = Gtk2::Gdk::Pixbuf->new_from_file("$sharedir/icons/gueststatus/Aborted.png");
+$gui{img}{DeletingSnapshot} = Gtk2::Gdk::Pixbuf->new_from_file("$sharedir/icons/gueststatus/Misc.png");
+$gui{img}{DeletingSnapshotOnline} = Gtk2::Gdk::Pixbuf->new_from_file("$sharedir/icons/gueststatus/Misc.png");
+$gui{img}{DeletingSnapshotPaused} = Gtk2::Gdk::Pixbuf->new_from_file("$sharedir/icons/gueststatus/Misc.png");
 $gui{img}{Discarding} = Gtk2::Gdk::Pixbuf->new_from_file("$sharedir/icons/gueststatus/Discarding.png");
 $gui{img}{Error} = Gtk2::Gdk::Pixbuf->new_from_file("$sharedir/icons/gueststatus/Error.png");
 $gui{img}{FaultTolerantSyncing} = Gtk2::Gdk::Pixbuf->new_from_file("$sharedir/icons/gueststatus/Misc.png");
+$gui{img}{FirstOnline} = Gtk2::Gdk::Pixbuf->new_from_file("$sharedir/icons/gueststatus/Running.png");
+$gui{img}{FirstTransient} = Gtk2::Gdk::Pixbuf->new_from_file("$sharedir/icons/gueststatus/Running.png");
+$gui{img}{LastOnline} = Gtk2::Gdk::Pixbuf->new_from_file("$sharedir/icons/gueststatus/Running.png");
+$gui{img}{LastTransient} = Gtk2::Gdk::Pixbuf->new_from_file("$sharedir/icons/gueststatus/Running.png");
 $gui{img}{LiveSnapshotting} = Gtk2::Gdk::Pixbuf->new_from_file("$sharedir/icons/gueststatus/Misc.png");
+$gui{img}{OnlineSnapshotting} = Gtk2::Gdk::Pixbuf->new_from_file("$sharedir/icons/gueststatus/Misc.png");
 $gui{img}{Paused} = Gtk2::Gdk::Pixbuf->new_from_file("$sharedir/icons/gueststatus/Paused.png");
 $gui{img}{PoweredOff} = Gtk2::Gdk::Pixbuf->new_from_file("$sharedir/icons/gueststatus/PoweredOff.png");
 $gui{img}{Restoring} = Gtk2::Gdk::Pixbuf->new_from_file("$sharedir/icons/gueststatus/Restoring.png");
+$gui{img}{RestoringSnapshot} = Gtk2::Gdk::Pixbuf->new_from_file("$sharedir/icons/gueststatus/Restoring.png");
 $gui{img}{Running} = Gtk2::Gdk::Pixbuf->new_from_file("$sharedir/icons/gueststatus/Running.png");
 $gui{img}{Saved} = Gtk2::Gdk::Pixbuf->new_from_file("$sharedir/icons/gueststatus/Saved.png");
 $gui{img}{Saving} = Gtk2::Gdk::Pixbuf->new_from_file("$sharedir/icons/gueststatus/Saving.png");
+$gui{img}{SettingUp} = Gtk2::Gdk::Pixbuf->new_from_file("$sharedir/icons/gueststatus/Running.png");
+$gui{img}{Snapshotting} = Gtk2::Gdk::Pixbuf->new_from_file("$sharedir/icons/gueststatus/Misc.png");
 $gui{img}{Starting} = Gtk2::Gdk::Pixbuf->new_from_file("$sharedir/icons/gueststatus/Running.png");
+$gui{img}{Stopping} = Gtk2::Gdk::Pixbuf->new_from_file("$sharedir/icons/gueststatus/Misc.png");
 $gui{img}{Stuck} = Gtk2::Gdk::Pixbuf->new_from_file("$sharedir/icons/gueststatus/Stuck.png");
 $gui{img}{Teleported} = Gtk2::Gdk::Pixbuf->new_from_file("$sharedir/icons/gueststatus/PoweredOff.png");
-$gui{img}{TeleportIn} = Gtk2::Gdk::Pixbuf->new_from_file("$sharedir/icons/gueststatus/Misc.png");
+$gui{img}{Teleporting} = Gtk2::Gdk::Pixbuf->new_from_file("$sharedir/icons/gueststatus/Misc.png");
+$gui{img}{TeleportingIn} = Gtk2::Gdk::Pixbuf->new_from_file("$sharedir/icons/gueststatus/Misc.png");
 $gui{img}{TeleportingPausedVM} = Gtk2::Gdk::Pixbuf->new_from_file("$sharedir/icons/gueststatus/Paused.png");
-$gui{img}{DeletingSnapshotOnline} = Gtk2::Gdk::Pixbuf->new_from_file("$sharedir/icons/gueststatus/Misc.png");
-$gui{img}{DeletingSnapshotPaused} = Gtk2::Gdk::Pixbuf->new_from_file("$sharedir/icons/gueststatus/Misc.png");
-$gui{img}{RestoringSnapshot} = Gtk2::Gdk::Pixbuf->new_from_file("$sharedir/icons/gueststatus/Restoring.png");
-$gui{img}{DeletingSnapshot} = Gtk2::Gdk::Pixbuf->new_from_file("$sharedir/icons/gueststatus/Misc.png");
-$gui{img}{SettingUp} = Gtk2::Gdk::Pixbuf->new_from_file("$sharedir/icons/gueststatus/Running.png");
-$gui{img}{FirstOnline} = Gtk2::Gdk::Pixbuf->new_from_file("$sharedir/icons/gueststatus/Running.png");
-$gui{img}{LastOnline} = Gtk2::Gdk::Pixbuf->new_from_file("$sharedir/icons/gueststatus/Running.png");
-$gui{img}{FirstTransient} = Gtk2::Gdk::Pixbuf->new_from_file("$sharedir/icons/gueststatus/Running.png");
-$gui{img}{LastTransient} = Gtk2::Gdk::Pixbuf->new_from_file("$sharedir/icons/gueststatus/Running.png");
 # Storage Icons
 $gui{img}{Network} = Gtk2::Gdk::Pixbuf->new_from_file("$sharedir/icons/nw_16px.png");
 $gui{img}{HardDisk} = Gtk2::Gdk::Pixbuf->new_from_file("$sharedir/icons/hd_16px.png");
@@ -65,6 +69,7 @@ $gui{img}{ctr}{SATA} = Gtk2::Gdk::Pixbuf->new_from_file("$sharedir/icons/sata_16
 $gui{img}{ctr}{SAS} = $gui{img}{ctr}{SATA};
 $gui{img}{ctr}{SCSI} = Gtk2::Gdk::Pixbuf->new_from_file("$sharedir/icons/scsi_16px.png");
 $gui{img}{ctr}{Floppy} = Gtk2::Gdk::Pixbuf->new_from_file("$sharedir/icons/floppy_16px.png");
+$gui{img}{ctr}{USB} = Gtk2::Gdk::Pixbuf->new_from_file("$sharedir/icons/usb_16px.png");
 # Snapshot icons
 $gui{img}{SnapshotOffline} = Gtk2::Gdk::Pixbuf->new_from_file("$sharedir/icons/snapshot_offline_16px.png");
 $gui{img}{SnapshotOnline} = Gtk2::Gdk::Pixbuf->new_from_file("$sharedir/icons/snapshot_online_16px.png");
@@ -76,6 +81,18 @@ $gui{img}{VMGroup} = Gtk2::Gdk::Pixbuf->new_from_file("$sharedir/icons/vm_group_
 $gui{img}{DirIcon} = Gtk2::Gdk::Pixbuf->new_from_file("$sharedir/icons/sf_16px.png");
 $gui{img}{FileIcon} = Gtk2::Gdk::Pixbuf->new_from_file("$sharedir/icons/vm_open_filemanager_16px.png");
 $gui{img}{ParentIcon} = Gtk2::Gdk::Pixbuf->new_from_file("$sharedir/icons/sf_parent_16px.png");
+# Details Icons
+$gui{img}{CatGen} = Gtk2::Gdk::Pixbuf->new_from_file("$sharedir/icons/machine_16px.png");
+$gui{img}{CatSys} = Gtk2::Gdk::Pixbuf->new_from_file("$sharedir/icons/chipset_16px.png");
+$gui{img}{CatDisp} = Gtk2::Gdk::Pixbuf->new_from_file("$sharedir/icons/vrdp_16px.png");
+$gui{img}{CatStor} = Gtk2::Gdk::Pixbuf->new_from_file("$sharedir/icons/hd_16px.png");
+$gui{img}{CatAudio} = Gtk2::Gdk::Pixbuf->new_from_file("$sharedir/icons/sound_16px.png");
+$gui{img}{CatNet} = Gtk2::Gdk::Pixbuf->new_from_file("$sharedir/icons/nw_16px.png");
+$gui{img}{CatIO} = Gtk2::Gdk::Pixbuf->new_from_file("$sharedir/icons/serial_port_16px.png");
+$gui{img}{CatUSB} = Gtk2::Gdk::Pixbuf->new_from_file("$sharedir/icons/usb_16px.png");
+$gui{img}{CatShare} = Gtk2::Gdk::Pixbuf->new_from_file("$sharedir/icons/sf_16px.png");
+$gui{img}{CatDesc} = Gtk2::Gdk::Pixbuf->new_from_file("$sharedir/icons/description_16px.png");
+
 our %prefs;
 
 # Fill %gui so we can reference them easily.
@@ -87,23 +104,25 @@ foreach ($builder->get_objects) {
 
 # We have to register signals manually, which we want to block at some point because we need the sigids
 # as blocking any other way is not supported in perl-gtk (other blocking funcs map to null funcs)
-our %signal = (fam          => $gui{comboboxNewOSFam}->signal_connect(changed => \&newgen_osfam, $gui{comboboxNewOSVer}),
-               ver          => $gui{comboboxNewOSVer}->signal_connect(changed => \&newgen_osver, $gui{comboboxNewOSFam}),
-               famedit      => $gui{comboboxEditGenOSFam}->signal_connect(changed => \&gen_osfam, $gui{comboboxEditGenOSVer}),
-               veredit      => $gui{comboboxEditGenOSVer}->signal_connect(changed => \&gen_osver, $gui{comboboxEditGenOSFam}),
-               audiodrv     => $gui{comboboxEditAudioDriver}->signal_connect(changed => \&audio_driver),
-               audioctr     => $gui{comboboxEditAudioCtr}->signal_connect(changed => \&audio_ctr),
-               netadaptype  => $gui{comboboxEditNetType}->signal_connect(changed => \&net_adapter_type, $gui{checkbuttonEditNetEnable}),
-               netattach    => $gui{comboboxEditNetAttach}->signal_connect(changed => \&net_attach, $gui{checkbuttonEditNetEnable}),
-               netname      => $gui{comboboxEditNetName}->signal_connect(changed => \&net_name, $gui{checkbuttonEditNetEnable}),
-               genericdrv   => $gui{comboboxentryEditNetGenDriver}->signal_connect(changed => \&net_generic_driver, $gui{checkbuttonEditNetEnable}),
-               nameint      => $gui{comboboxentryEditNetNameInt}->signal_connect(changed => \&net_name_internal, $gui{checkbuttonEditNetEnable}),
-               stortype     => $gui{comboboxEditStorCtrType}->signal_connect(changed => \&storage_ctrtype),
-               ohcitoggle   => $gui{checkbuttonEditUSBEnable}->signal_connect(toggled => \&usb_toggle),
-               ehcitoggle   => $gui{checkbuttonEditUSBEHCI}->signal_connect(toggled => \&usb_toggleehci));
+our %signal = (fam                => $gui{comboboxNewOSFam}->signal_connect(changed => \&newgen_osfam, $gui{comboboxNewOSVer}),
+               ver                => $gui{comboboxNewOSVer}->signal_connect(changed => \&newgen_osver, $gui{comboboxNewOSFam}),
+               famedit            => $gui{comboboxEditGenOSFam}->signal_connect(changed => \&gen_osfam, $gui{comboboxEditGenOSVer}),
+               veredit            => $gui{comboboxEditGenOSVer}->signal_connect(changed => \&gen_osver, $gui{comboboxEditGenOSFam}),
+               audiodrv           => $gui{comboboxEditAudioDriver}->signal_connect(changed => \&audio_driver),
+               audioctr           => $gui{comboboxEditAudioCtr}->signal_connect(changed => \&audio_ctr),
+               netadaptype        => $gui{comboboxEditNetType}->signal_connect(changed => \&net_adapter_type, $gui{checkbuttonEditNetEnable}),
+               netattach          => $gui{comboboxEditNetAttach}->signal_connect(changed => \&net_attach, $gui{checkbuttonEditNetEnable}),
+               netname            => $gui{comboboxEditNetName}->signal_connect(changed => \&net_name, $gui{checkbuttonEditNetEnable}),
+               genericdrv         => $gui{comboboxentryEditNetGenDriver}->signal_connect(changed => \&net_generic_driver, $gui{checkbuttonEditNetEnable}),
+               nameint            => $gui{comboboxentryEditNetNameInt}->signal_connect(changed => \&net_name_internal, $gui{checkbuttonEditNetEnable}),
+               stortype           => $gui{comboboxEditStorCtrType}->signal_connect(changed => \&storage_ctrtype),
+               usbtoggle          => $gui{checkbuttonEditUSBEnable}->signal_connect(toggled => \&usb_toggle),
+               snapfolderactivate => $gui{entryEditGenSnapFolder}->signal_connect(activate => \&gen_snapfolder),
+               snapfolderfocus    => $gui{entryEditGenSnapFolder}->signal_connect(activate => \&gen_snapfolder));
 
 # Work around a stupid bug in Glade 3.10.x that disables toolbutton menus
 $gui{toolbuttonStop}->set_menu($gui{menuStop});
+$gui{toolbuttonCAD}->set_menu($gui{menuKeyboardMini});
 
 # Work around another bug in Glade, setting selection mode does not work
 $gui{treeviewInfo}->get_selection->set_mode('GTK_SELECTION_NONE');
@@ -230,6 +249,7 @@ sub handle_vboxfilechooser {
 # Handle select the appliance file to export to from the file chooser
 sub handle_exportapplfilechooser {
     my ($basedir, $filename) = @_;
+
     if ($basedir and $filename) {
         $filename .= '.ova' if ($filename !~ m/.ovf$/i and $filename !~ m/.ova$/i);
         $gui{entryExportApplFile}->set_text(&rcatfile($basedir, $filename));
@@ -240,33 +260,7 @@ sub handle_exportapplfilechooser {
 sub handle_importapplfilechooser {
     my ($basedir, $filename) = @_;
 
-    if ($basedir and $filename) {
-        my $appliancefile = &rcatfile($basedir, $filename);
-        &addrow_log("Importing appliance from $appliancefile");
-        my $IAppliance = IVirtualBox_createAppliance($gui{websn});
-        my $IProgress = IAppliance_read($IAppliance, $appliancefile);
-        
-        if ($IProgress) {
-            &show_progress_window($IProgress, 'Reading Appliance');
-            IAppliance_interpret($IAppliance);
-            my $warnings = IAppliance_getWarnings($IAppliance);
-            &addrow_log("Import warnings: $warnings") if ($warnings);
-            $IProgress = IAppliance_importMachines($IAppliance);
-            
-            if ($IProgress) {
-                &show_progress_window($IProgress, 'Importing Appliance') if ($IProgress);
-                my @appluuid = IAppliance_getMachines($IAppliance);
-                foreach my $id (@appluuid) {
-                    &makesel_list_guest($id); # Make the current selection the new appliance
-                    &fill_list_guest(); # Repopulate the guest list so it appears and is selected
-                    &addrow_log("Imported new appliance from $appliancefile");
-                    &show_dialog_edit(); # Edit any settings
-                }
-            }
-            else { &show_err_msg('applimport', "($appliancefile)"); }
-        }
-        else { &show_err_msg('applimport', "($appliancefile)"); }
-    }
+    $gui{entryImportApplFile}->set_text(&rcatfile($basedir, $filename)) if ($basedir and $filename);
 }
 
 
@@ -474,16 +468,6 @@ sub show_progress_window() {
     return $resultcode;
 }
 
-# Save RemoteBox's window position for restoration later
-sub save_window_pos() {
-    my ($winname) = @_;
-    my $alloc = $gui{$winname}->allocation;
-    my ($w, $h) = ($alloc->width, $alloc->height);
-    my ($x,$y) = $gui{$winname}->get_position();
-    $prefs{"WINPOS_$winname"} = "$w:$h:$x:$y";
-    &rbprefs_save();
-}
-
 # Restore RemoteBox's window position to the last save position
 sub restore_window_pos() {
     my ($winname) = @_;
@@ -496,7 +480,7 @@ sub restore_window_pos() {
 }
 
 # Permit certain chars only for a guest name
-sub validate_name() {
+sub validate_name {
     my ($entry, $char, $len, $pos) = @_;
     $char =~ s/[\?\/\;\*\\\<\>\|\.]//; # Strip these chars
     return $char, $pos;
@@ -510,55 +494,60 @@ sub validate_group {
 }
 
 # Basic character validation for hexadecial
-sub validate_hex() {
+sub validate_hex {
     my ($entry, $char, $len, $pos) = @_;
     $char =~ s/[^A-F0-9a-f]//; # Strip everything but these chars
     return $char, $pos;
 }
 
 # Basic character validation for a port range
-sub validate_port() {
+sub validate_port {
     my ($entry, $char, $len, $pos) = @_;
     $char =~ s/[^0-9,-]//; # Strip everything but these chars
     return $char, $pos;
 }
 
 # Permit only numbers in an entry
-sub validate_number() {
+sub validate_number {
     my ($entry, $char, $len, $pos) = @_;
     $char =~ s/[^0-9]//; # Strip everything but these chars
     return $char, $pos;
 }
 
 # Basic character validation for IPv4
-sub validate_ip() {
+sub validate_ipv4 {
     my ($entry, $char, $len, $pos) = @_;
     $char =~ s/[^0-9,.]//; # Strip everything but these chars
     return $char, $pos;
 }
 
 # Basic character validation for IPv6
-sub validate_ipv6() {
+sub validate_ipv6 {
     my ($entry, $char, $len, $pos) = @_;
     $char =~ s/[^A-F0-9a-f,:]//; # Strip everything but these chars
     return $char, $pos;
 }
 
 # Basic character validation for CIDR
-sub validate_cidr() {
+sub validate_cidr {
     my ($entry, $char, $len, $pos) = @_;
     $char =~ s/[^0-9,.,\/]//; # Strip everything but these chars
     return $char, $pos;
 }
 
-sub valid_cidr() {
+# Is it valid IPv4 CIDR format
+sub valid_cidr {
     return 1 if ($_[0] =~ m/^(([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\.){3}([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])(\/(\d|[1-2]\d|3[0-2]))$/);
 }
 
-sub valid_ipv4() {
+# Is it a valid IPv4 address
+sub valid_ipv4 {
     return 1 if ($_[0] =~ m/^(([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\.){3}([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])$/);
 }
 
+# Is it a valid IPv6 address. Needs fixing
+sub valid_ipv6 { return 1; }
+
 sub combobox_set_active_text {
     my ($combobox, $text) = @_;
     my $i = -1;
