@@ -3,138 +3,205 @@ use strict;
 use warnings;
 our (%gui, %vmc, %signal, %prefs);
 
-sub fill_list_vminfo() {
-    my $vhost = &vhost();
-    $gui{liststoreInfo}->clear();
-    my $gref = &getsel_list_guest();
-    &addrow_log("Fetching more details of $$gref{Name}...");
-    my $IVRDEServer = IMachine_getVRDEServer($$gref{IMachine});
-    my @IStorageController = IMachine_getStorageControllers($$gref{IMachine});
-    my $IAudioAdapter = IMachine_getAudioAdapter($$gref{IMachine});
-    my @IUSBController = IMachine_getUSBControllers($$gref{IMachine});
-    &addrow_info(0, 'GENERAL', 2, 800);
-    &addrow_info(0, 'Name:', 1, $$gref{Name});
-    &addrow_info(0, 'OS Type:', 1, $$gref{Os});
-    &addrow_info(0, 'SYSTEM', 2, 800);
-    &addrow_info(0, 'Firmware:', 1, IMachine_getFirmwareType($$gref{IMachine}));
-    my $mem = IMachine_getMemorySize($$gref{IMachine});
-    $mem = ($mem > 1023) ? sprintf("%.2f GB", $mem / 1024) : "$mem MB";
-    &addrow_info(0, 'Base Memory:', 1, $mem);
-    &addrow_info(0, 'Processor(s):', 1, IMachine_getCPUCount($$gref{IMachine}));
-    my $bootorder = '';
+# Block for filling in guest details
+{
 
-    foreach (1..4) {
-        my $bdev = IMachine_getBootOrder($$gref{IMachine}, $_);
-       $bootorder .= "$bdev  " if ($bdev ne 'Null');
+    # This two subs might be able to be replaced with a single one
+    sub expand_details_row {
+        my ($treeview, $treeiter, $treepath) = @_;
+        my $model = $treeview->get_model();
+        my @row = $model->get($treeiter);
+        $prefs{$row[5]} = 1;
     }
- 
-    $bootorder ? &addrow_info(0, 'Boot Order:', 1, $bootorder) : &addrow_info(0, 'Boot Order:', 1, '<No boot devices enabled>');
-    my $vtx = '';
-    $vtx .= 'VT-x/AMD-V  ' if (IMachine_getHWVirtExProperty($$gref{IMachine}, 'Enabled') eq 'true');
-    $vtx .= 'VPID  ' if (IMachine_getHWVirtExProperty($$gref{IMachine}, 'VPID') eq 'true');
-    $vtx .= 'PAE/NX  ' if (IMachine_getCPUProperty($$gref{IMachine}, 'PAE') eq 'true');
-    $vtx .= 'Nested Paging  ' if (IMachine_getHWVirtExProperty($$gref{IMachine}, 'NestedPaging') eq 'true');
-    $vtx ? &addrow_info(0, 'Acceleration:', 1, $vtx) : &addrow_info(0, 'Acceleration:', 1, '<No acceleration enabled>');
-    &addrow_info(0, 'DISPLAY', 2, 800);
-    &addrow_info(0, 'Video Memory:', 1, IMachine_getVRAMSize($$gref{IMachine}) . ' MB');
-    &addrow_info(0, 'Screens: ', 1, IMachine_getMonitorCount($$gref{IMachine}));
-    my $vidaccel = '';
-    $vidaccel .= '2D Video  ' if (IMachine_getAccelerate2DVideoEnabled($$gref{IMachine}) eq 'true');
-    $vidaccel .= '3D  ' if (IMachine_getAccelerate3DEnabled($$gref{IMachine}) eq 'true');
-    $vidaccel ? &addrow_info(0, 'Display Acceleration:', 1, $vidaccel) : &addrow_info(0, 'Display Acceleration:', 1, '<No video acceleration enabled>');
-    IVRDEServer_getEnabled($IVRDEServer) eq 'true' ? &addrow_info(0, 'Remote Display Port(s):', 1, IVRDEServer_getVRDEProperty($IVRDEServer, 'TCP/Ports'))
-                                                   : &addrow_info(0, 'Remote Display Port(s):', 1, '<RDP server disabled>');
-    &addrow_info(0, 'STORAGE', 2, 800);
 
-    foreach my $controller (@IStorageController) {
-        my $controllername = IStorageController_getName($controller);
-        &addrow_info(0, 'Controller:', 1, $controllername);
-        my @IMediumAttachment = IMachine_getMediumAttachmentsOfController($$gref{IMachine}, $controllername);
-        foreach my $attachment (@IMediumAttachment) {
-            if ($$attachment{medium}) {
-                IMedium_refreshState($$attachment{medium}); # Needed to bring in current sizes
-                # Use the base medium for information purposes
-                my $size = &bytesToX(IMedium_getLogicalSize($$attachment{medium}));
-                &addrow_info(0, "  Port $$attachment{port}:", 1, IMedium_getName(IMedium_getBase($$attachment{medium})) . " ($$attachment{type}, $size)");
+    sub collapse_details_row {
+        my ($treeview, $treeiter, $treepath) = @_;
+        my $model = $treeview->get_model();
+        my @row = $model->get($treeiter);
+        $prefs{$row[5]} = 0;
+    }
+
+    # Fill a brief version of the guest details
+    sub fill_list_details_brief {
+        &addrow_log("Fetching guest details...");
+        $gui{treestoreDetails}->clear();
+        my $treemodel = $gui{treeviewDetails}->get_model();
+        my $gref = &getsel_list_guest();
+        my $iter = &addrow_details(undef, 0, $gui{img}{CatGen}, 1, 'Guest Summary', 3, 800, 4, 0.0, 5, 'EXPANDDETGEN');
+        &addrow_details($iter, 1, ' Name:', 2, $$gref{Name});
+        &addrow_details($iter, 1, ' Operating System:', 2, IMachine_getOSTypeId($$gref{IMachine}));
+        my $mem = IMachine_getMemorySize($$gref{IMachine});
+        $mem = ($mem > 1023) ? sprintf("%.2f GB", $mem / 1024) : "$mem MB";
+        &addrow_details($iter, 1, ' Base Memory:', 2, $mem);
+        &addrow_details($iter, 1, ' Video Memory:', 2, IMachine_getVRAMSize($$gref{IMachine}) . ' MB');
+        $gui{treeviewDetails}->expand_row($treemodel->get_path($iter), 1) if ($prefs{EXPANDDETGEN});
+        my $desciter = &addrow_details(undef, 0, $gui{img}{CatDesc}, 1, 'Description', 3, 800, 4, 0.0, 5, 'EXPANDDETDESC');
+        my $desc = IMachine_getDescription($$gref{IMachine});
+        $desc ? &addrow_details($desciter, 1, ' Description:', 2, $desc)
+            : &addrow_details($desciter, 1, ' <None>');
+        $gui{treeviewDetails}->expand_row($treemodel->get_path($desciter), 1) if ($prefs{EXPANDDETDESC});
+        &addrow_log("Guest details retrieved.");
+    }
+
+    # Fill the guest details
+    sub fill_list_details {
+        &addrow_log("Fetching extended guest details...");
+        my $vhost = &vhost();
+        $gui{treestoreDetails}->clear();
+        my $treemodel = $gui{treeviewDetails}->get_model();
+        my $gref = &getsel_list_guest();
+        my $IVRDEServer = IMachine_getVRDEServer($$gref{IMachine});
+        my @IStorageController = IMachine_getStorageControllers($$gref{IMachine});
+        my $IAudioAdapter = IMachine_getAudioAdapter($$gref{IMachine});
+        my @IUSBController = IMachine_getUSBControllers($$gref{IMachine});
+        my $geniter = &addrow_details(undef, 0, $gui{img}{CatGen}, 1, 'General', 3, 800, 4, 0.0, 5, 'EXPANDDETGEN');
+        &addrow_details($geniter, 1, ' Name:', 2, $$gref{Name});
+        &addrow_details($geniter, 1, ' Operating System:', 2, IMachine_getOSTypeId($$gref{IMachine}));
+        $gui{treeviewDetails}->expand_row($treemodel->get_path($geniter), 1) if ($prefs{EXPANDDETGEN});
+
+        my $sysiter = &addrow_details(undef, 0, $gui{img}{CatSys}, 1, 'System', 3, 800, 4, 0.0, 5, 'EXPANDDETSYS');
+        my $mem = IMachine_getMemorySize($$gref{IMachine});
+        $mem = ($mem > 1023) ? sprintf("%.2f GB", $mem / 1024) : "$mem MB";
+        &addrow_details($sysiter, 1, ' Base Memory:', 2, $mem);
+        &addrow_details($sysiter, 1, ' Firmware:', 2, IMachine_getFirmwareType($$gref{IMachine}));
+        &addrow_details($sysiter, 1, ' Processors:', 2, IMachine_getCPUCount($$gref{IMachine}));
+        my $bootorder = '';
+
+        foreach (1..4) {
+            my $bdev = IMachine_getBootOrder($$gref{IMachine}, $_);
+        $bootorder .= "$bdev  " if ($bdev ne 'Null');
+        }
+    
+        $bootorder ? &addrow_details($sysiter, 1, ' Boot Order:', 2, $bootorder) : &addrow_details($sysiter, 1, ' Boot Order:', 2, '<None Enabled>');
+        my $vtx = '';
+        $vtx .= 'VT-x/AMD-V  ' if (IMachine_getHWVirtExProperty($$gref{IMachine}, 'Enabled') eq 'true');
+        $vtx .= 'VPID  ' if (IMachine_getHWVirtExProperty($$gref{IMachine}, 'VPID') eq 'true');
+        $vtx .= 'PAE/NX  ' if (IMachine_getCPUProperty($$gref{IMachine}, 'PAE') eq 'true');
+        $vtx .= 'Nested Paging  ' if (IMachine_getHWVirtExProperty($$gref{IMachine}, 'NestedPaging') eq 'true');
+        $vtx ? &addrow_details($sysiter, 1, ' Acceleration:', 2, $vtx) : &addrow_details($sysiter, 1, ' Acceleration:', 2, '<None Enabled>');
+        &addrow_details($sysiter, 1, ' Paravirtualization:', 2, IMachine_getParavirtProvider($$gref{IMachine}));
+        $gui{treeviewDetails}->expand_row($treemodel->get_path($sysiter), 1) if ($prefs{EXPANDDETSYS});
+
+        my $dispiter = &addrow_details(undef, 0, $gui{img}{CatDisp}, 1, 'Display', 3, 800, 4, 0.0, 5, 'EXPANDDETDISP');
+        &addrow_details($dispiter, 1, ' Video Memory:', 2, IMachine_getVRAMSize($$gref{IMachine}) . ' MB');
+        &addrow_details($dispiter, 1, ' Screens: ', 2, IMachine_getMonitorCount($$gref{IMachine}));
+        my $vidaccel = '';
+        $vidaccel .= '2D Video  ' if (IMachine_getAccelerate2DVideoEnabled($$gref{IMachine}) eq 'true');
+        $vidaccel .= '3D  ' if (IMachine_getAccelerate3DEnabled($$gref{IMachine}) eq 'true');
+        $vidaccel ? &addrow_details($dispiter, 1, ' Acceleration:', 2, $vidaccel) : &addrow_details($dispiter, 1, ' Acceleration:', 2, '<None Enabled>');
+        IVRDEServer_getEnabled($IVRDEServer) eq 'true' ? &addrow_details($dispiter, 1, ' Remote Display Ports:', 2, IVRDEServer_getVRDEProperty($IVRDEServer, 'TCP/Ports'))
+                                                    : &addrow_details($dispiter, 1, ' Remote Display Ports:', 2, '<Remote Display Disabled>');
+        $gui{treeviewDetails}->expand_row($treemodel->get_path($dispiter), 1) if ($prefs{EXPANDDETDISP});
+
+        my $storiter = &addrow_details(undef, 0, $gui{img}{CatStor}, 1, 'Storage', 3, 800, 4, 0.0, 5, 'EXPANDDETSTOR');
+        foreach my $controller (@IStorageController) {
+            my $controllername = IStorageController_getName($controller);
+            &addrow_details($storiter, 1, ' Controller:', 2, $controllername);
+            my @IMediumAttachment = IMachine_getMediumAttachmentsOfController($$gref{IMachine}, $controllername);
+            foreach my $attachment (@IMediumAttachment) {
+                if ($$attachment{medium}) {
+                    IMedium_refreshState($$attachment{medium}); # Needed to bring in current sizes
+                    # Use the base medium for information purposes
+                    my $size = &bytesToX(IMedium_getLogicalSize($$attachment{medium}));
+                    &addrow_details($storiter, 1, "   Port $$attachment{port}:", 2, IMedium_getName(IMedium_getBase($$attachment{medium})) . " ($$attachment{type}, $size)");
+                }
             }
         }
-    }
 
-    &addrow_info(0, 'AUDIO', 2, 800);
-    IAudioAdapter_getEnabled($IAudioAdapter) eq 'true' ? (&addrow_info(0, 'Host Driver:', 1, IAudioAdapter_getAudioDriver($IAudioAdapter))
-                                                       and &addrow_info(0, 'Controller:', 1, IAudioAdapter_getAudioController($IAudioAdapter)))
-                                                       : &addrow_info(0, '<Audio Disabled>');
-    &addrow_info(0, 'NETWORK', 2, 800);
+        $gui{treeviewDetails}->expand_row($treemodel->get_path($storiter), 1) if ($prefs{EXPANDDETSTOR});
 
-    foreach (0..($$vhost{maxnet}-1)) {
-        my $INetworkAdapter = IMachine_getNetworkAdapter($$gref{IMachine}, $_);
+        my $audioiter = &addrow_details(undef, 0, $gui{img}{CatAudio}, 1, 'Audio', 3, 800, 4, 0.0, 5, 'EXPANDDETAUDIO');
+        IAudioAdapter_getEnabled($IAudioAdapter) eq 'true' ? (&addrow_details($audioiter, 1, ' Host Driver:', 2, IAudioAdapter_getAudioDriver($IAudioAdapter))
+                                                        and &addrow_details($audioiter, 1, ' Controller:', 2, IAudioAdapter_getAudioController($IAudioAdapter)))
+                                                        : &addrow_details($audioiter, 1, ' <Audio Disabled>');
+        $gui{treeviewDetails}->expand_row($treemodel->get_path($audioiter), 1) if ($prefs{EXPANDDETAUDIO});
 
-        if (INetworkAdapter_getEnabled($INetworkAdapter) eq 'true') {
-            my $attachtype = INetworkAdapter_getAttachmentType($INetworkAdapter);
-            my $adapter = INetworkAdapter_getAdapterType($INetworkAdapter) . ' (' . $attachtype;
+        my $netiter = &addrow_details(undef, 0, $gui{img}{CatNet}, 1, 'Network', 3, 800, 4, 0.0, 5, 'EXPANDDETNET');
+        foreach (0..($$vhost{maxnet}-1)) {
+            my $INetworkAdapter = IMachine_getNetworkAdapter($$gref{IMachine}, $_);
 
-            if ($attachtype eq 'Bridged') { $adapter .= ', ' . INetworkAdapter_getBridgedInterface($INetworkAdapter); }
-            elsif ($attachtype eq 'HostOnly') { $adapter .= ', ' . INetworkAdapter_getHostOnlyInterface($INetworkAdapter); }
-            elsif ($attachtype eq 'Internal') { $adapter .= ', ' . INetworkAdapter_getInternalNetwork($INetworkAdapter); }
+            if (INetworkAdapter_getEnabled($INetworkAdapter) eq 'true') {
+                my $attachtype = INetworkAdapter_getAttachmentType($INetworkAdapter);
+                my $adapter = INetworkAdapter_getAdapterType($INetworkAdapter) . ' (' . $attachtype;
 
-            $adapter .= ')';
-            &addrow_info(0, "Adapter $_:", 1, $adapter);
-        }
-    }
+                if ($attachtype eq 'Bridged') { $adapter .= ', ' . INetworkAdapter_getBridgedInterface($INetworkAdapter); }
+                elsif ($attachtype eq 'HostOnly') { $adapter .= ', ' . INetworkAdapter_getHostOnlyInterface($INetworkAdapter); }
+                elsif ($attachtype eq 'Internal') { $adapter .= ', ' . INetworkAdapter_getInternalNetwork($INetworkAdapter); }
 
-    &addrow_info(0, 'SERIAL PORTS', 2, 800);
-
-    foreach (0..($$vhost{maxser}-1)) {
-        my $ISerialPort = IMachine_getSerialPort($$gref{IMachine}, $_);
-        ISerialPort_getEnabled($ISerialPort) eq 'true' ? &addrow_info(0, "Port #$_:", 1, 'Enabled  ' . 
-                                                                                         ISerialPort_getHostMode($ISerialPort) . '  ' .
-                                                                                         ISerialPort_getPath($ISerialPort))
-                                                       : &addrow_info(0, "Port #$_:", 1, 'Disabled');
-    }
-
-    &addrow_info(0, 'PARALLEL PORT', 2, 800);
-    my $IParallelPort = IMachine_getParallelPort($$gref{IMachine}, 0);
-    IParallelPort_getEnabled($IParallelPort) eq 'true' ? &addrow_info(0, 'LPT:', 1, 'Enabled  ' . IParallelPort_getPath($IParallelPort))
-                                                       : &addrow_info(0, 'Disabled');
-
-    &addrow_info(0, 'USB', 2, 800);
-
-    if (@IUSBController) {
-        foreach my $usbcontroller (@IUSBController) {
-            my $usbver = IUSBController_getUSBStandard($usbcontroller);
-            my $major = int(IUSBController_getUSBStandard($usbcontroller) / 256);
-            my $minor = IUSBController_getUSBStandard($usbcontroller) % 256;
-            &addrow_info(0, 'Controller:',
-                         1, IUSBController_getName($usbcontroller) . 
-                            ' (' . IUSBController_getType($usbcontroller) . ', ' .
-                            'USB v' . int($usbver/256) . '.' . $usbver % 256 . ')');
+                $adapter .= ')';
+                &addrow_details($netiter, 1, " Adapter $_:", 2, $adapter);
+            }
         }
 
-        my $IUSBDeviceFilters = IMachine_getUSBDeviceFilters($$gref{IMachine});
-        my @filters = IUSBDeviceFilters_getDeviceFilters($IUSBDeviceFilters);
-        my $active = 0;
-        foreach (@filters) { $active++ if (IUSBDeviceFilter_getActive($_) eq 'true'); }
-        &addrow_info(0, 'Device Filters:', 1,  scalar(@filters) . " ($active active)");
+        $gui{treeviewDetails}->expand_row($treemodel->get_path($netiter), 1) if ($prefs{EXPANDDETNET});
+
+        my $ioiter = &addrow_details(undef, 0, $gui{img}{CatIO}, 1, 'I/O Ports', 3, 800, 4, 0.0, 5, 'EXPANDDETIO');
+        foreach (0..($$vhost{maxser}-1)) {
+            my $ISerialPort = IMachine_getSerialPort($$gref{IMachine}, $_);
+            ISerialPort_getEnabled($ISerialPort) eq 'true' ? &addrow_details($ioiter, 1, " Serial Port #$_:", 2, 'Enabled  ' . 
+                                                                                            ISerialPort_getHostMode($ISerialPort) . '  ' .
+                                                                                            ISerialPort_getPath($ISerialPort))
+                                                        : &addrow_details($ioiter, 1, " Serial Port #$_:", 2, 'Disabled');
+        }
+
+        my $IParallelPort = IMachine_getParallelPort($$gref{IMachine}, 0);
+        IParallelPort_getEnabled($IParallelPort) eq 'true' ? &addrow_details($ioiter, 1, ' LPT Port:', 2, 'Enabled  ' . IParallelPort_getPath($IParallelPort))
+                                                        : &addrow_details($ioiter, 1, ' LPT Port:', 2, 'Disabled');
+        $gui{treeviewDetails}->expand_row($treemodel->get_path($ioiter), 1) if ($prefs{EXPANDDETIO});
+
+        my $usbiter = &addrow_details(undef, 0, $gui{img}{CatUSB}, 1, 'USB', 3, 800, 4, 0.0, 5, 'EXPANDDETUSB');
+        if (@IUSBController) {
+            foreach my $usbcontroller (@IUSBController) {
+                my $usbver = IUSBController_getUSBStandard($usbcontroller);
+                &addrow_details($usbiter, 1, ' Controller:',
+                                        2, IUSBController_getName($usbcontroller) . 
+                                        ' (' . IUSBController_getType($usbcontroller) . ')');
+            }
+
+            my $IUSBDeviceFilters = IMachine_getUSBDeviceFilters($$gref{IMachine});
+            my @filters = IUSBDeviceFilters_getDeviceFilters($IUSBDeviceFilters);
+            my $active = 0;
+            foreach (@filters) { $active++ if (IUSBDeviceFilter_getActive($_) eq 'true'); }
+            &addrow_details($usbiter, 1, '  Device Filters:', 2,  scalar(@filters) . " ($active active)");
+        }
+        else { &addrow_details($usbiter, 1, ' <None Enabled>'); }
+        $gui{treeviewDetails}->expand_row($treemodel->get_path($usbiter), 1) if ($prefs{EXPANDDETUSB});
+
+        my $shareiter = &addrow_details(undef, 0, $gui{img}{CatShare}, 1, 'Shared Folders', 3, 800, 4, 0.0, 5, 'EXPANDDETSHARE');
+        my @sf = IMachine_getSharedFolders($$gref{IMachine});
+        &addrow_details($shareiter, 1, ' Shared Folders:', 2, scalar(@sf));
+        $gui{treeviewDetails}->expand_row($treemodel->get_path($shareiter), 1) if ($prefs{EXPANDDETSHARE});
+
+        my $sref = &get_session($$gref{IMachine});
+
+        if ($$sref{Lock} eq 'Shared') {
+            my $runiter = &addrow_details(undef, 0, $gui{img}{CatGen}, 1, 'Runtime Details', 3, 800, 4, 0.0, 5, 'EXPANDDETRUN');
+            my $IGuest = IConsole_getGuest(ISession_getConsole($$sref{ISession}));
+            &addrow_details($runiter, 1, ' OS:', 2, IGuest_getOSTypeId($IGuest));
+            my $additionsversion = IGuest_getAdditionsVersion($IGuest);
+            if ($additionsversion) { &addrow_details($runiter, 1, ' Guest Additions:', 2, $additionsversion); }
+            else { &addrow_details($runiter, 1, ' Guest Additions:', 2, 'Not Installed (or not running)'); }
+            $gui{treeviewDetails}->expand_row($treemodel->get_path($runiter), 1) if ($prefs{EXPANDDETRUN});
+        }
+
+        ISession_unlockMachine($$sref{ISession}) if (ISession_getState($$sref{ISession}) eq 'Locked');
+
+        my $desciter = &addrow_details(undef, 0, $gui{img}{CatDesc}, 1, 'Description', 3, 800, 4, 0.0, 5, 'EXPANDDETDESC');
+        my $desc = IMachine_getDescription($$gref{IMachine});
+        $desc ? &addrow_details($desciter, 1, ' Description:', 2, $desc)
+            : &addrow_details($desciter, 1, ' <None>');
+        $gui{treeviewDetails}->expand_row($treemodel->get_path($desciter), 1) if ($prefs{EXPANDDETDESC});
+
+        &addrow_log("Extended guest details retrieved.");
     }
-    else { &addrow_info(0, 'None'); }
+}
 
-    my @sf = IMachine_getSharedFolders($$gref{IMachine});
-    &addrow_info(0, 'SHARED FOLDERS', 2, 800);
-    &addrow_info(0, 'Shared Folders:', 1, scalar(@sf));
-
-    my $sref = &get_session($$gref{IMachine});
-
-    if ($$sref{Lock} eq 'Shared') {
-        &addrow_info(0, 'RUNTIME INFORMATION', 2, 800);
-        my $IGuest = IConsole_getGuest(ISession_getConsole($$sref{ISession}));
-        &addrow_info(0, 'Operating System:', 1, IGuest_getOSTypeId($IGuest));
-        my $additionsversion = IGuest_getAdditionsVersion($IGuest);
-        if ($additionsversion) { &addrow_info(0, 'Guest Additions:', 1, $additionsversion); }
-        else { &addrow_info(0, 'Guest Additions:', 1, 'Not Installed (or have not loaded yet)'); }
-    }
-
-    &addrow_log('More details complete.');
-    ISession_unlockMachine($$sref{ISession}) if (ISession_getState($$sref{ISession}) eq 'Locked');
+# Adds a line to the details view
+sub addrow_details {
+    my $iter = shift;
+    my $citer = $gui{treestoreDetails}->append($iter);
+    $gui{treestoreDetails}->set($citer, @_);
+    return $citer;
 }
 
 # Fills the remote file chooser with a list of files. Involves a lot of splicing because
@@ -268,26 +335,19 @@ sub fill_list_serverinfo() {
     &addrow_info(0, 'Autostart DB:', 1, $$vhost{autostartdb});
 }
 
-# Populate the shared folder list for editing guest settings
-sub fill_list_editshared() {
+# Populate the permanent and transient shared folder list for the guest settings
+sub fill_list_editshared {
     my ($IMachine) = @_;
+    my $sref = &get_session($IMachine);
+    my @ISharedFolderPerm = IMachine_getSharedFolders($IMachine);
+    my $IConsole = ISession_getConsole($$sref{ISession});
+    my @ISharedFolderTran = IConsole_getSharedFolders($IConsole) if ($IConsole);
     $gui{buttonEditSharedRemove}->set_sensitive(0);
     $gui{buttonEditSharedEdit}->set_sensitive(0);
-    $gui{liststoreEditShared}->clear();
-    my @ISharedFolder = IMachine_getSharedFolders($IMachine);
+    $gui{liststoreEditShared}->clear(); 
 
-    foreach my $share (@ISharedFolder) {
-        my $access = 'Full';
-        my $automount = 'No';
-        my $tooltip = "$$share{name}: $$share{hostPath}";
-        $access = 'Read-Only' if ($$share{writable} eq 'false');
-        $automount = 'Yes' if ($$share{autoMount} eq 'true');
-        $tooltip = $$share{lastAccessError} if ($$share{lastAccessError});
-        $tooltip = 'Share is not accessible' if ($$share{accessible} eq 'false');
-        my $iter = $gui{liststoreEditShared}->append;
-        if ($$share{accessible} eq 'false') { $gui{liststoreEditShared}->set($iter, 0, $$share{name}, 1, $$share{hostPath}, 2, $access, 3, $automount, 4, $gui{img}{Error}, 5, $tooltip); }
-        else { $gui{liststoreEditShared}->set($iter, 0, $$share{name}, 1, $$share{hostPath}, 2, $access, 3, $automount, 5, $tooltip); }
-    }
+    foreach (@ISharedFolderPerm) { &addrow_editshared($_, 'Yes'); }
+    foreach (@ISharedFolderTran) { &addrow_editshared($_, 'No'); }
 }
 
 # Populates the guest's storage list
@@ -455,6 +515,21 @@ sub addrow_log() {
     $gui{treeviewLog}->scroll_to_cell($gui{treeviewLog}->get_model->get_path($iter));
 }
 
+# Adds a row to the editshared list
+sub addrow_editshared {
+    my ($share, $permanent) = @_;
+    my $access = 'Full';
+    my $automount = 'No';
+    my $tooltip = "$$share{name}: $$share{hostPath}";
+    $access = 'Read-Only' if ($$share{writable} eq 'false');
+    $automount = 'Yes' if ($$share{autoMount} eq 'true');
+    $tooltip = $$share{lastAccessError} if ($$share{lastAccessError});
+    $tooltip = 'Share is not accessible' if ($$share{accessible} eq 'false');
+    my $iter = $gui{liststoreEditShared}->append;
+    if ($$share{accessible} eq 'false') { $gui{liststoreEditShared}->set($iter, 0, $$share{name}, 1, $$share{hostPath}, 2, $access, 3, $automount, 4, $gui{img}{Error}, 5, $tooltip, 6, $permanent); }
+    else { $gui{liststoreEditShared}->set($iter, 0, $$share{name}, 1, $$share{hostPath}, 2, $access, 3, $automount, 5, $tooltip, 6, $permanent); }
+}
+
 sub addrow_info() {
     my $iter = $gui{liststoreInfo}->append;
     $gui{liststoreInfo}->set($iter, @_);
@@ -503,7 +578,7 @@ sub getsel_list_editshared() {
     return undef if (!$iter);
     my @row = $model->get($iter);
     my %hash;
-    $hash{$_} = shift @row foreach ('Name', 'Folder', 'Access', 'Mount', 'Accessible', 'Tooltip');
+    $hash{$_} = shift @row foreach ('Name', 'Folder', 'Access', 'Mount', 'Accessible', 'Tooltip', 'Permanent');
     return \%hash;
 }
 
@@ -618,6 +693,10 @@ sub onsel_list_editstorage() {
             &sens_editstor_ctr();
             $gui{comboboxEditStorCtrType}->set_model($gui{liststoreEditStorSASCtrType});
         }
+        elsif ($$storref{Bus} eq 'USB') {
+            &sens_editstor_ctr();
+            $gui{comboboxEditStorCtrType}->set_model($gui{liststoreEditStorUSBCtrType});
+        }
         else { # Defaults to SCSI
             &sens_editstor_ctr();
             $gui{comboboxEditStorCtrType}->set_model($gui{liststoreEditStorSCSICtrType});
@@ -865,16 +944,16 @@ sub onsel_list_editstorage() {
     }
 }
 
-# Port Forwarding List Handling
+# IPv4 Port Forwarding List Handling
 {
     my %selected = (Name => '');
 
-    sub getsel_list_pf { return \%selected; }
+    sub getsel_list_pf4 { return \%selected; }
 
-    sub fill_list_pf {
+    sub fill_list_pf4 {
         my ($INATNetwork) = @_;
         &busy_pointer($gui{dialogNATDetails}, 1);
-        &clr_list_pf();
+        &clr_list_pf4();
         my @rules = INATNetwork_getPortForwardRules4($INATNetwork);
         foreach my $rule (@rules) {
             my ($rname, $rproto, $rhip, $rhport, $rgip, $rgport) = split ':', $rule;
@@ -885,27 +964,70 @@ sub onsel_list_editstorage() {
 
             if ($rname eq $selected{Name}) {
                 $gui{treeviewPFRulesIPv4}->get_selection()->select_iter($iter);
-                &onsel_list_pf();
+                &onsel_list_pf4();
             }
         }
         &busy_pointer($gui{dialogNATDetails}, 0);
     }
 
-    sub onsel_list_pf {
+    sub onsel_list_pf4 {
         my $model = $gui{treeviewPFRulesIPv4}->get_model();
         my $iter = $gui{treeviewPFRulesIPv4}->get_selection->get_selected() ? $gui{treeviewPFRulesIPv4}->get_selection->get_selected() : $model->get_iter_first();
         my @row = $model->get($iter);
         $selected{$_} = shift @row foreach ('Name', 'Protocol', 'HostIP', 'HostPort', 'GuestIP', 'GuestPort', 'INATNetwork');
-        $gui{buttonPFRulesRemove}->set_sensitive(1);
+        $gui{buttonPFRulesRemove4}->set_sensitive(1);
     }
 
-    sub clr_list_pf {
+    sub clr_list_pf4 {
         $gui{liststorePFRulesIPv4}->clear();
-        $gui{buttonPFRulesRemove}->set_sensitive(0);
+        $gui{buttonPFRulesRemove4}->set_sensitive(0);
     }
 
 }
 
+# IPv6 Port Forwarding List Handling
+{
+    my %selected = (Name => '');
+
+    sub getsel_list_pf6 { return \%selected; }
+
+    sub fill_list_pf6 {
+        my ($INATNetwork) = @_;
+        &busy_pointer($gui{dialogNATDetails}, 1);
+        &clr_list_pf6();
+        my @rules = INATNetwork_getPortForwardRules6($INATNetwork);
+        foreach my $rule (@rules) {
+            # Jump through hoops because VB decided to use : as a column separator! Doh!
+            $rule =~ s/\[(.*?)\]//;
+            my $rhip = $1;
+            $rule =~ s/\[(.*?)\]//;
+            my $rgip = $1;
+            my ($rname, $rproto, undef, $rhport, undef, $rgport) = split ':', $rule;
+            my $iter = $gui{liststorePFRulesIPv6}->append;
+            $gui{liststorePFRulesIPv6}->set($iter, 0, $rname, 1, uc($rproto), 2, $rhip, 3, $rhport, 4, $rgip, 5, $rgport, 6, $INATNetwork);
+
+            if ($rname eq $selected{Name}) {
+                $gui{treeviewPFRulesIPv6}->get_selection()->select_iter($iter);
+                &onsel_list_pf6();
+            }
+        }
+        &busy_pointer($gui{dialogNATDetails}, 0);
+    }
+
+    sub onsel_list_pf6 {
+        my $model = $gui{treeviewPFRulesIPv6}->get_model();
+        my $iter = $gui{treeviewPFRulesIPv6}->get_selection->get_selected() ? $gui{treeviewPFRulesIPv6}->get_selection->get_selected() : $model->get_iter_first();
+        my @row = $model->get($iter);
+        $selected{$_} = shift @row foreach ('Name', 'Protocol', 'HostIP', 'HostPort', 'GuestIP', 'GuestPort', 'INATNetwork');
+        $gui{buttonPFRulesRemove6}->set_sensitive(1);
+    }
+
+    sub clr_list_pf6 {
+        $gui{liststorePFRulesIPv6}->clear();
+        $gui{buttonPFRulesRemove6}->set_sensitive(0);
+    }
+
+}
 
 # VMM HD List Handling
 {
@@ -1031,6 +1153,11 @@ sub clr_list_vmm() {
     $treestore->clear();
 }
 
+sub onsel_list_shared {
+    $gui{buttonEditSharedRemove}->set_sensitive(1);
+    $gui{buttonEditSharedEdit}->set_sensitive(1);
+}
+
 # Snapshot List Handling
 {
     my %selected = (ISnapshot => '');
@@ -1062,7 +1189,6 @@ sub clr_list_vmm() {
     sub fill_list_snapshots() {
         my $gref = &getsel_list_guest();
         &addrow_log("Fetching snapshots for $$gref{Name}...");
-        &busy_pointer($gui{windowMain}, 1);
         &clr_list_snapshots();
 
         if (IMachine_getSnapshotCount($$gref{IMachine}) > 0) {
@@ -1072,7 +1198,6 @@ sub clr_list_vmm() {
             $gui{treeviewSnapshots}->expand_all();
         }
 
-        &busy_pointer($gui{windowMain}, 0);
         &addrow_log('Snapshot fetch complete.');
     }
 
@@ -1114,7 +1239,8 @@ sub clr_list_vmm() {
 
     sub getsel_list_guest() { return \%selected; }
 
-    sub onsel_list_guest() {
+    sub onsel_list_guest {
+        &busy_pointer($gui{windowMain}, 1);
         my $model = $gui{treeviewGuest}->get_model();
         my $iter = $gui{treeviewGuest}->get_selection->get_selected() ? $gui{treeviewGuest}->get_selection->get_selected() : $model->get_iter_first();
         my @row = $model->get($iter);
@@ -1122,12 +1248,13 @@ sub clr_list_vmm() {
         # If there's no IMachine, it's a group so don't waste anymore time
         if (!$row[2]) { 
             &sens_unselected();
-            &clr_summarydetails();
+            $gui{treestoreDetails}->clear();
+            &busy_pointer($gui{windowMain}, 0);
             return;
         }
 
         $selected{$_} = shift @row foreach ('Name', 'Os', 'IMachine', 'Status', 'Osid', 'Uuid', 'Icon', 'Prettyname', 'Statusicon');
-        &fill_summarydetails();
+        $prefs{EXTENDEDDETAILS} ? &fill_list_details() : &fill_list_details_brief();
         &sens_unselected();
         &fill_list_snapshots();
         my $status = IMachine_getState($selected{IMachine});
@@ -1143,9 +1270,9 @@ sub clr_list_vmm() {
             $gui{menuitemDisplay}->set_sensitive(1);
             $gui{menuitemLogs}->set_sensitive(1);
             $gui{toolbuttonStop}->set_sensitive(1);
+            $gui{toolbuttonCAD}->set_sensitive(1);
             $gui{toolbuttonReset}->set_sensitive(1);
             $gui{toolbuttonRemoteDisplay}->set_sensitive(1);
-            $gui{buttonShowDetails}->set_sensitive(1);
             $gui{toolbuttonSettings}->set_sensitive(1);         # Online editing
             $gui{buttonRefreshSnapshot}->set_sensitive(1);
             $gui{buttonTakeSnapshot}->set_sensitive(1);
@@ -1169,7 +1296,6 @@ sub clr_list_vmm() {
             $gui{menuitemUngroup}->set_sensitive(1);
             $gui{toolbuttonStart}->set_sensitive(1);
             $gui{toolbuttonDiscard}->set_sensitive(1);
-            $gui{buttonShowDetails}->set_sensitive(1);
             $gui{buttonRefreshSnapshot}->set_sensitive(1);
             $gui{buttonTakeSnapshot}->set_sensitive(1);
         }
@@ -1179,10 +1305,11 @@ sub clr_list_vmm() {
             $gui{menuitemRemoteDisplay}->set_sensitive(1);
             $gui{menuitemLogs}->set_sensitive(1);
             $gui{toolbuttonRemoteDisplay}->set_sensitive(1);
-            $gui{buttonShowDetails}->set_sensitive(1);
+            $gui{buttonRefreshSnapshot}->set_sensitive(1);
+            $gui{buttonTakeSnapshot}->set_sensitive(1);
         }
         elsif ($status eq 'PoweredOff' | $status eq 'Aborted') {
-            $gui{menuitemExportAppliance}->set_sensitive(1);
+            $gui{menuitemExportAppl}->set_sensitive(1);
             $gui{menuitemAction}->set_sensitive(1);
             $gui{menuitemStart}->set_sensitive(1);
             $gui{menuitemSettings}->set_sensitive(1);
@@ -1193,12 +1320,17 @@ sub clr_list_vmm() {
             $gui{menuitemLogs}->set_sensitive(1);
             $gui{toolbuttonStart}->set_sensitive(1);
             $gui{toolbuttonSettings}->set_sensitive(1);
-            $gui{buttonShowDetails}->set_sensitive(1);
             $gui{buttonRefreshSnapshot}->set_sensitive(1);
             $gui{buttonTakeSnapshot}->set_sensitive(1);
             $gui{menuitemHotPlugCPU}->set_sensitive(1) if (&bl(IMachine_getCPUHotPlugEnabled($selected{IMachine})));
         }
+        elsif ($status eq 'Stuck') {
+               &sens_unselected();
+               $gui{toolbuttonStop}->set_sensitive(1);
+        }
         else { &sens_unselected(); }
+
+        &busy_pointer($gui{windowMain}, 0);
     }
 
     sub add_guest_group() {
@@ -1287,7 +1419,7 @@ sub clr_list_vmm() {
     sub clr_list_guest() {
         &sens_unselected();
         $gui{treestoreGuest}->clear();
-        &clr_summarydetails();
+        $gui{treestoreDetails}->clear();
         &clr_list_snapshots();
     }
 }
