@@ -1,19 +1,36 @@
 # Main Entry for RemoteBox
 use strict;
 use warnings;
-$0 = 'RemoteBox VirtualBox Client';
 our (%gui, %prefs, $sharedir, $docdir);
+my %cmdopts=();
 
 $endpoint = 'http://localhost:18083';
 $fault = sub{}; # Do nothing with faults until connected
 &rbprefs_get();
-$gui{expanderMessages}->set_expanded($prefs{MSGLOGEXPAND});
 &restore_window_pos('windowMain');
-&addrow_log("Welcome to $gui{appname} $gui{appver} (for use with VirtualBox 4.3.x)");
+&addrow_log("Welcome to $gui{appname} $gui{appver} (for use with VirtualBox 5.0.x)");
+getopts("H:u:p:h", \%cmdopts);
+
+if($cmdopts{h}) {
+    print STDERR <<USAGE;
+Usage:
+    $0 [-h]
+    $0 [-H <host>] [-u <user>] [-p <pass>]
+
+    -h        : Help
+    -H <host> : Automatically connect to this virtualbox host
+    -u <user> : Connect using this username. If omitted an empty username is assumed. Only useful with -H
+    -p <pass> : Connect using this password. If omitted an empty password is assumed. Only useful with -H
+USAGE
+    exit 0;
+}
+elsif ($cmdopts{H}) { &show_dialog_connect('AUTO'); }
+
+$0 = 'RemoteBox VirtualBox Client';
 Gtk2->main;
 
 sub quit_remotebox() {
-    &save_window_pos('windowMain');
+    &save_window_pos('windowMain'); # Also saves the main preferences
     &virtualbox_logoff();
 
     # These should be reset on exit to help garbage collection
@@ -75,6 +92,9 @@ sub virtualbox_logoff() {
 
 # Be careful with use of $vhost in this sub!
 sub show_dialog_connect() {
+    my ($widget) = @_;
+    my $response = 'cancel';
+
     $gui{liststoreConnectURL}->clear();
     $gui{liststoreConnectUser}->clear();
     $gui{checkbuttonConnectSave}->set_active($prefs{SAVEURLUSER});
@@ -90,14 +110,37 @@ sub show_dialog_connect() {
         $gui{liststoreConnectUser}->set($iter, 0, $_);
     }
 
-    my $response = $gui{dialogConnect}->run;
-    $gui{dialogConnect}->hide;
+    # Check if a command line automatic login has been requested. We do this here
+    # to try an ensure that a cmdline login is as close as possible to a GUI login
+    if ($widget eq 'AUTO') {
+        my $iter = $gui{liststoreConnectURL}->append();
+        $gui{liststoreConnectURL}->set($iter, 0, $cmdopts{H});
+        $gui{comboboxentryConnectURL}->set_active_iter($iter);
+
+        if ($cmdopts{u}) {
+            $iter = $gui{liststoreConnectUser}->append();
+            $gui{liststoreConnectUser}->set($iter, 0, $cmdopts{u});
+            $gui{comboboxentryConnectUser}->set_active_iter($iter);
+        }
+
+        if ($cmdopts{p}) { $gui{entryConnectPassword}->set_text($cmdopts{p}); }
+
+        $response = 'ok';
+        &addrow_log("Attempting automatic login to $endpoint");
+    }
+    else {
+        # Normal GUI login
+        $response = $gui{dialogConnect}->run;
+        $gui{dialogConnect}->hide;
+    }
+
     $gui{dialogConnect}->get_display->flush;
 
     if ($response eq 'ok') {
         my $url = $gui{comboboxentryConnectURL}->get_active_text();
         my $user = $gui{comboboxentryConnectUser}->get_active_text();
         $url = $endpoint if (!$url);
+        $url =~ s/(\/)+$//g; # Remove all trailing 
         $url = "http://$url" if ($url !~ m/^.+:\/\//);
         $url = "$url:18083" if ($url !~ m/:\d+$/);
 
@@ -119,7 +162,7 @@ sub show_dialog_connect() {
                 my $ISystemProperties = IVirtualBox_getSystemProperties($gui{websn});
                 &addrow_log("Logged onto $endpoint");
                 &addrow_log("Running VirtualBox $ver.");
-                &show_err_msg('vboxver', "\nDetected VirtualBox Version: $ver") if ($ver !~ m/^4.3/);
+                &show_err_msg('vboxver', "\nDetected VirtualBox Version: $ver") if ($ver !~ m/^5.0/);
                 &show_err_msg('noextensions') if (ISystemProperties_getDefaultVRDEExtPack($ISystemProperties) !~ m/Oracle VM VirtualBox Extension Pack/i);
                 &fill_list_guest();
                 &sens_connect(1);
@@ -171,15 +214,6 @@ sub show_dialog_log() {
     $gui{dialogLog}->hide;
 }
 
-
-sub show_dialog_vminfo() {
-    &fill_list_vminfo();
-    my $gref = &getsel_list_guest();
-    $gui{dialogInfo}->set_title("$$gref{Name} : More Details");
-    $gui{dialogInfo}->run;
-    $gui{dialogInfo}->hide;
-}
-
 # Show the export appliance dialog
 sub show_dialog_exportappl {
     my $gref = &getsel_list_guest();
@@ -194,7 +228,7 @@ sub show_dialog_exportappl {
 
         if ($response eq 'ok') {
             # Other entries don't required validation
-            if (!$gui{entryExportApplFile}->get_text()) { &show_err_msg('invalidfile') ; }
+            if (!$gui{entryExportApplFile}->get_text()) { &show_err_msg('invalidfile'); }
             elsif (!$gui{entryExportApplName}->get_text()) { &show_err_msg('invalidname'); }
             else {
                 $gui{dialogExportAppl}->hide;
@@ -225,35 +259,7 @@ sub show_dialog_exportappl {
     } until (!$gui{dialogExportAppl}->visible());
 }
 
-sub fill_summarydetails {
-    my $gref = &getsel_list_guest();
-    &addrow_log("Fetching details summary for $$gref{Name}...");
-    $gui{labelDetailsName}->set_markup('<b>Name:</b> ' . $$gref{Name});
-    my $osid = IMachine_getOSTypeId($$gref{IMachine});
-    $gui{labelDetailsOSType}->set_markup('<b>OS Type:</b> ' . $$gref{Os});
-    my $mem = IMachine_getMemorySize($$gref{IMachine});
-    $mem = ($mem > 1023) ? sprintf("%.2f GB", $mem / 1024) : "$mem MB";
-    $gui{labelDetailsBaseMem}->set_markup("<b>Base Memory:</b> $mem");
-    $gui{labelDetailsVidMem}->set_markup('<b>Video Memory:</b> ' . IMachine_getVRAMSize($$gref{IMachine}) . ' MB');
-
-    IMachine_getHWVirtExProperty($$gref{IMachine}, 'Enabled') eq 'true' ? $gui{labelDetailsVTX}->set_markup('<b>VT-x/AMD-V:</b> Enabled')
-                                                                        : $gui{labelDetailsVTX}->set_markup('<b>VT-x/AMD-V:</b> Disabled');
-
-    my $desc = IMachine_getDescription($$gref{IMachine});
-    $desc ? $gui{labelDetailsDescription}->set_text($desc) : $gui{labelDetailsDescription}->set_text('<No Description>');
-    &addrow_log('Details summary complete.');
-}
-
-sub clr_summarydetails() {
-    $gui{labelDetailsName}->set_text('');
-    $gui{labelDetailsOSType}->set_text('');
-    $gui{labelDetailsBaseMem}->set_text('');
-    $gui{labelDetailsVidMem}->set_text('');
-    $gui{labelDetailsVTX}->set_text('');
-    $gui{labelDetailsDescription}->set_text('');
-}
-
-sub show_dialog_snapshotdetails() {
+sub show_dialog_snapshotdetails {
     my $snapref = &getsel_list_snapshots();
 
     if ($$snapref{ISnapshot}) { # This is needed in case a user double clicks on "Current State" which is not a real snapshot
@@ -290,8 +296,8 @@ sub show_dialog_snapshot() {
 sub discard_saved_state() {
     my $gref = &getsel_list_guest();
     my $sref = &get_session($$gref{IMachine});
-    my $IConsole = ISession_getConsole($$sref{ISession});
-    IConsole_discardSavedState($IConsole, 'true');
+    
+    IMachine_discardSavedState($$sref{IMachine}, 'true');
     &fill_list_guest();
     &addrow_log("Discarded saved execution state of $$gref{Name}.");
 
@@ -360,9 +366,8 @@ sub stop_guest_savestate() {
     my $sref = &get_session($$gref{IMachine});
 
     if ($$sref{Lock} eq 'Shared') {
-        my $IConsole = ISession_getConsole($$sref{ISession});
-
-        if (my $IProgress = IConsole_saveState($IConsole)) { # Not cancellable
+        
+        if (my $IProgress = IMachine_saveState($$sref{IMachine})) { # Not cancellable
             &show_progress_window($IProgress, "Saving guest execution state of $$gref{Name}");
             &fill_list_guest();
             &addrow_log("Saved the execution state of $$gref{Name}.");
@@ -461,7 +466,7 @@ sub restore_snapshot() {
 
     if ($$sref{Type} eq 'WriteLock') { # Not cancellable
         my $snapref = &getsel_list_snapshots();
-        my $IProgress = IConsole_restoreSnapshot(ISession_getConsole($$sref{ISession}), $$snapref{ISnapshot});
+        my $IProgress = IMachine_restoreSnapshot($$sref{IMachine}, $$snapref{ISnapshot});
         &show_progress_window($IProgress, 'Restoring snapshot');
         &addrow_log("Snapshot of $$gref{Name} restored.");
     }
@@ -482,7 +487,7 @@ sub delete_snapshot() {
         if (ISnapshot_getChildrenCount($$snapref{ISnapshot}) > 1) { &show_err_msg('snapdelchild', " ($$gref{Name})"); }
         else { # Not cancellable
             my $snapuuid = ISnapshot_getId($$snapref{ISnapshot});
-            my $IProgress = IConsole_deleteSnapshot(ISession_getConsole($$sref{ISession}), $snapuuid);
+            my $IProgress = IMachine_deleteSnapshot($$sref{IMachine}, $snapuuid);
             &show_progress_window($IProgress, 'Deleting snapshot') if ($IProgress);
             &addrow_log("Snapshot of $$gref{Name} deleted.");
         }
@@ -501,7 +506,7 @@ sub take_snapshot() {
     my $sref = &get_session($$gref{IMachine});
 
     if ($$sref{Lock} ne 'None') { # Not cancellable
-        my $IProgress = IConsole_takeSnapshot(ISession_getConsole($$sref{ISession}), $name, $description);
+        my ($snapid, $IProgress) = IMachine_takeSnapshot($$sref{IMachine}, $name, $description, 'true');
         &show_progress_window($IProgress, 'Taking snapshot') if ($IProgress);
         &addrow_log("Created a new snapshot of $$gref{Name}.");
     }
@@ -1026,16 +1031,16 @@ sub keyboard_send() {
         $sequence = 'Alt-SysRq+H';
         @scancodes = (56, 84, 184, 212, 35, 163); # Actually sends Alt-SysRq THEN H
     }
-    elsif ($widget eq $gui{menuitemKeyboardCABS}) {
+    elsif ($widget eq $gui{menuitemKeyboardCABS} or $widget eq $gui{menuitemKeyboardMiniCABS}) {
         $sequence = 'Ctrl-Alt-Backspace';
         @scancodes = (29, 56, 14, 157, 184, 142);
     }
-    elsif ($widget eq $gui{menuitemKeyboardCTRLC}) {
-        $sequence = 'Ctrl-c';
+    elsif ($widget eq $gui{menuitemKeyboardCTRLC} or $widget eq $gui{menuitemKeyboardMiniCTRLC}) {
+        $sequence = 'Ctrl-C';
         @scancodes = (29, 46, 157, 174);
     }
-    elsif ($widget eq $gui{menuitemKeyboardCTRLD}) {
-        $sequence = 'Ctrl-d';
+    elsif ($widget eq $gui{menuitemKeyboardCTRLD} or $widget eq $gui{menuitemKeyboardMiniCTRLD}) {
+        $sequence = 'Ctrl-D';
         @scancodes = (29, 32, 157, 160);
     }
 
@@ -1160,7 +1165,7 @@ sub sens_connect() {
     my ($state) = @_;
     $gui{menuitemNew}->set_sensitive($state);
     $gui{menuitemAdd}->set_sensitive($state);
-    $gui{menuitemImportAppliance}->set_sensitive($state);
+    $gui{menuitemImportAppl}->set_sensitive($state);
     $gui{menuitemVMM}->set_sensitive($state);
     $gui{menuitemServerInfo}->set_sensitive($state);
     $gui{menuitemVBPrefs}->set_sensitive($state);
@@ -1171,7 +1176,7 @@ sub sens_connect() {
 
 # Sets the sensitivity when no guest is selected
 sub sens_unselected {
-    $gui{menuitemExportAppliance}->set_sensitive(0);
+    $gui{menuitemExportAppl}->set_sensitive(0);
     $gui{menuitemAction}->set_sensitive(0);
     $gui{menuitemStart}->set_sensitive(0);
     $gui{menuitemStop}->set_sensitive(0);
@@ -1196,9 +1201,9 @@ sub sens_unselected {
     $gui{toolbuttonStop}->set_sensitive(0);
     $gui{toolbuttonSettings}->set_sensitive(0);
     $gui{toolbuttonRemoteDisplay}->set_sensitive(0);
+    $gui{toolbuttonCAD}->set_sensitive(0);
     $gui{toolbuttonDiscard}->set_sensitive(0);
     $gui{toolbuttonReset}->set_sensitive(0);
-    $gui{buttonShowDetails}->set_sensitive(0);
     $gui{buttonRefreshSnapshot}->set_sensitive(0);
     $gui{buttonCloneSnapshot}->set_sensitive(0);
     $gui{buttonTakeSnapshot}->set_sensitive(0);
@@ -1214,6 +1219,7 @@ sub reset_icon() {
 # Takes a PNG screenshot of the guest
 sub screenshot() {
     my ($widget) = @_;
+
     my $gref = &getsel_list_guest();
     my $sref = &get_session($$gref{IMachine});
     $gui{filechooserscreenshot}->set_current_name("$$gref{Name}_screenshot.png");
@@ -1228,7 +1234,7 @@ sub screenshot() {
             my $IConsole = ISession_getConsole($$sref{ISession});
             my $IDisplay = IConsole_getDisplay($IConsole);
             my ($w, $h, $d, $ox, $oy) = IDisplay_getScreenResolution($IDisplay, 0);
-            if (my $rawscreenshot = IDisplay_takeScreenShotPNGToArray($IDisplay, 0, $w, $h)) {
+            if (my $rawscreenshot = IDisplay_takeScreenShotToArray($IDisplay, 0, $w, $h, 'PNG')) {
                 my $screenshot = decode_base64($rawscreenshot);
                 if (open(SHOT, '>', $filename)) {
                     binmode SHOT; # Not normally needed for UNIX, but more portable
@@ -1256,7 +1262,7 @@ sub screenshot_to_icon() {
         my $IConsole = ISession_getConsole($$sref{ISession});
         my $IDisplay = IConsole_getDisplay($IConsole);
 
-        if (my $rawicon = IDisplay_takeScreenShotPNGToArray($IDisplay, 0, 32, 32)) {
+        if (my $rawicon = IDisplay_takeScreenShotToArray($IDisplay, 0, 32, 32, 'PNG')) {
             mkdir($gui{THUMBDIR}, 0755) unless (-e $gui{THUMBDIR});
             my $icon = decode_base64($rawicon);
             if (open(ICON, '>', "$gui{THUMBDIR}/$$gref{Uuid}.png")) {
@@ -1286,6 +1292,52 @@ sub group_list_collapse {
     &rbprefs_save();
 }
 
+# Shows the dialog for importing an appliance
+sub show_dialog_importappl {
+
+    do {
+        my $response = $gui{dialogImportAppl}->run;
+
+        if ($response eq 'ok') {
+            my $importopts = '';
+            $importopts = 'ImportToVDI' if ($gui{checkbuttonImportApplToVDI}->get_active());
+
+            if (!$gui{entryImportApplFile}->get_text()) { &show_err_msg('invalidfile'); }
+            else {
+                $gui{dialogImportAppl}->hide;
+                my $appliancefile = $gui{entryImportApplFile}->get_text();
+                &addrow_log("Importing appliance from $appliancefile");
+                my $IAppliance = IVirtualBox_createAppliance($gui{websn});
+                my $IProgress = IAppliance_read($IAppliance, $appliancefile);
+        
+                if ($IProgress) {
+                    &show_progress_window($IProgress, 'Reading Appliance');
+                    IAppliance_interpret($IAppliance);
+                    my $warnings = IAppliance_getWarnings($IAppliance);
+                    &addrow_log("Import warnings: $warnings") if ($warnings);
+                    $IProgress = IAppliance_importMachines($IAppliance, $importopts);
+            
+                    if ($IProgress) {
+                        &show_progress_window($IProgress, 'Importing Appliance') if ($IProgress);
+                        my @appluuid = IAppliance_getMachines($IAppliance);
+                        foreach my $id (@appluuid) {
+                            &makesel_list_guest($id); # Make the current selection the new appliance
+                            &fill_list_guest(); # Repopulate the guest list so it appears and is selected
+                            &addrow_log("Imported new appliance from $appliancefile");
+                            &show_dialog_edit(); # Edit any settings
+                        }
+                    }
+                    else { &show_err_msg('applimport', "($appliancefile)"); }
+                }
+                else { &show_err_msg('applimport', "($appliancefile)"); } 
+            }
+        }
+        else { $gui{dialogImportAppl}->hide; }
+
+    } until (!$gui{dialogImportAppl}->visible());
+}
+
+# Displays the dialog for setting a guests group
 sub show_dialog_group {
     my $gref = &getsel_list_guest();
     my $sref = &get_session($$gref{IMachine});
@@ -1351,6 +1403,16 @@ sub show_manual() {
     else { system(qq[xdg-open "$docdir/remotebox.pdf" &]); }
 }
 
+# Display extended details when a guest is selected
+sub extended_details {
+    $prefs{EXTENDEDDETAILS} = $gui{checkbuttonShowDetails}->get_active();
+    &rbprefs_save();
+    # Force a refresh if there's a selected guest
+    my $model = $gui{treeviewGuest}->get_model();
+    my $iter = $gui{treeviewGuest}->get_selection->get_selected() ? $gui{treeviewGuest}->get_selection->get_selected() : 0;
+    &onsel_list_guest() if ($iter); 
+}
+
 # Converts bytes into a human readable format with unit
 sub bytesToX() {
     my ($bytes) = @_;
