@@ -151,7 +151,7 @@ sub show_dialog_clone {
 # Determines the next free port number and device number on a controller. If
 # there isn't one, then a new one will be created, provided the controller is
 # not at its maximum. If it is -1 is returned for the port and device numbers
-sub get_free_deviceport() {
+sub get_free_deviceport {
     # !!Be careful about portnum versus portcount!! Eg ports 0 to 7 is a portcount of 8
     my ($IMachine, $IStorCtr) = @_;
 
@@ -194,7 +194,7 @@ sub get_free_deviceport() {
     return %address;
 }
 
-sub create_new_clone() {
+sub create_new_clone {
     my $vhost = &vhost();
     my ($srcgref, $clonename, $clonetype, $newmacs) = @_;
 
@@ -237,7 +237,7 @@ sub create_new_clone() {
 }
 
 # Creates a new guest
-sub create_new_guest() {
+sub create_new_guest {
     my ($newref) = @_;
     my $osver = &osver();
     my %os = %{ $$osver{$$newref{ver}} };
@@ -263,6 +263,7 @@ sub create_new_guest() {
         IMachine_addUSBController($IMachine, 'EHCI', 'EHCI') if (&bl($os{recommendedUSB}));
         IMachine_setPointingHIDType($IMachine, 'USBMouse') if ($os{recommendedUSBHID} eq 'true');
         IMachine_setPointingHIDType($IMachine, 'USBTablet') if ($os{recommendedUSBTablet} eq 'true');
+        IMachine_setAccelerate2DVideoEnabled($IMachine, $os{recommended2DVideoAcceleration});
         IAudioAdapter_setAudioController($IAudioAdapter, $os{recommendedAudioController});
         IAudioAdapter_setAudioDriver($IAudioAdapter, 'Null');
         IAudioAdapter_setEnabled($IAudioAdapter, 1);
@@ -296,12 +297,12 @@ sub create_new_guest() {
 }
 
 # Creates a new hard disk and shows a progress window
-sub create_new_hd() {
+sub create_new_hd {
     my ($newref) = @_;
     my $ext = 'hdd';
     # All formats use an extension the same as their name except parallels
     $ext = $$newref{imgformat} unless ($$newref{imgformat} eq 'parallels');
-    my $IMedium = IVirtualBox_createHardDisk($gui{websn}, $$newref{imgformat}, &rcatfile($$newref{location}, "$$newref{diskname}.$ext"));
+    my $IMedium = IVirtualBox_createMedium($gui{websn}, $$newref{imgformat}, &rcatfile($$newref{location}, "$$newref{diskname}.$ext"), 'ReadWrite', 'HardDisk');
     my $IProgress = IMedium_createBaseStorage($IMedium, $$newref{size}, $$newref{allocation});
     &show_progress_window($IProgress, 'Creating Hard Disk');
 
@@ -316,7 +317,7 @@ sub create_new_hd() {
     return $IMedium;
 }
 
-sub newgen_osfam() {
+sub newgen_osfam {
     my ($combofam, $combover) = @_;
     my ($osfam, $osver) = &osfamver();
     my $fam = &getsel_combo($combofam, 1);
@@ -341,7 +342,7 @@ sub newgen_osfam() {
     $combover->signal_emit('changed'); # Force update of other fields based on OS
 }
 
-sub newgen_osver() {
+sub newgen_osver {
     my ($combover, $combofam) = @_;
     my $osver = &osver();
     my $ver = &getsel_combo($combover, 1);
@@ -353,7 +354,7 @@ sub newgen_osver() {
     $combover->signal_handler_unblock($signal{ver});
 }
 
-sub newstor_new_exist() {
+sub newstor_new_exist {
     my ($widget) = @_;
     my $buttongrp = $widget->get_group();
 
@@ -368,7 +369,7 @@ sub newstor_new_exist() {
 }
 
 # Handle the toggle startup disk selection
-sub toggle_newstartupdisk() {
+sub toggle_newstartupdisk {
     if ($gui{checkbuttonNewNewHD}->get_active() == 1) {
         $gui{radiobuttonNewNewHD}->show();
         $gui{radiobuttonNewExistingHD}->show();
@@ -384,14 +385,14 @@ sub toggle_newstartupdisk() {
 }
 
 # Handle the generate new MACs depending on clone type
-sub clone_type() {
+sub clone_type {
     if ($gui{comboboxCloneType}->get_active() == 2) { $gui{checkbuttonCloneNewMACs}->hide(); }
     else { $gui{checkbuttonCloneNewMACs}->show(); }
 }
 
 # Handle the radio button sensitivity when selecting an image format when
 # creating a new hd for a new guest
-sub sens_hdformatchanged() {
+sub sens_hdformatchanged {
     my $format = &getsel_combo($gui{comboboxNewNewFormat}, 1);
     $gui{radiobuttonNewDynamic}->set_active(1);
 
